--_________________________________________________________________________________________________
--                                                                                                |
--                                           |TDC core|                                           |
--                                                                                                |
--                                         CERN,BE/CO-HT                                          |
--________________________________________________________________________________________________|

---------------------------------------------------------------------------------------------------
--                                                                                                |
--                                      fmc_masterFIP_core                                        |
--                                                                                                |
---------------------------------------------------------------------------------------------------
-- File         fmc_masterFIP_core.vhd                                                            |
--                                                                                                |
-- Description  The masterFIP core top level instantiates all the modules needed to provide
--                                                                                                |
-- Authors      Evangelia Gousiou (Evangelia.Gousiou@cern.ch)                                     |
-- Date         07/2014                                                                           |
-- Version      v1                                                                                |
-- Depends on                                                                                     |
--                                                                                                |
----------------                                                                                  |
-- Last changes                                                                                   |
--     07/2014  v1  EG  First version                                                             |
--                                                                                                |
---------------------------------------------------------------------------------------------------

---------------------------------------------------------------------------------------------------
--                               GNU LESSER GENERAL PUBLIC LICENSE                                |
--                              ------------------------------------                              |
-- This source file is free software; you can redistribute it and/or modify it under the terms of |
-- the GNU Lesser General Public License as published by the Free Software Foundation; either     |
-- version 2.1 of the License, or (at your option) any later version.                             |
-- This source is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;       |
-- without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.      |
-- See the GNU Lesser General Public License for more details.                                    |
-- You should have received a copy of the GNU Lesser General Public License along with this       |
-- source; if not, download it from http://www.gnu.org/licenses/lgpl-2.1.html                     |
---------------------------------------------------------------------------------------------------

--=================================================================================================
--                                       Libraries & Packages
--=================================================================================================
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;
use work.masterFIP_pkg.all;
use work.gencores_pkg.all;
use work.wishbone_pkg.all;
use work.wrcore_pkg.all;
use work.genram_pkg.all;


--=================================================================================================
--                                Entity declaration for fmc_masterFIP_core
--=================================================================================================
entity fmc_masterFIP_core is
  generic
    (g_span             : integer := 32;     -- address span in bus interfaces
     g_width            : integer := 32;     -- data width in bus interfaces
     values_for_simul   : boolean := FALSE); -- this generic is set to TRUE when instantiated in a test-bench
  port
    (-- Clock and reset
     clk_i              : in    std_logic;   -- 40 MHz clock
     rst_i              : in    std_logic;   -- PoR or PCIe reset, synched with the clk_i

     -- Bus Speed                            -- 31.25 Kbps: speed_b1 = 0, speed_b0 = 0
     speed_b0_i         : in    std_logic;   -- 1 Mbps    : speed_b1 = 0, speed_b0 = 1
     speed_b1_i         : in    std_logic;   -- 2.5 Mbps  : speed_b1 = 1, speed_b0 = 0
                                             -- 5 Mbps    : speed_b1 = 1, speed_b0 = 1
     -- Bus termination
     bus_term_en_n_o    : out std_logic;     -- 150 Ohms termination of the bus -- check again
     
     -- External synchronisation pulse transceiver 
     ext_sync_term_en_o : out std_logic;     -- enable 50 Ohms termination of the pulse
     ext_sync_dir_o     : out std_logic;     -- transceiver direction 
     ext_sync_oe_o      : out std_logic;     -- transceiver output enable
     ext_sync_tst_n_o   : out std_logic;     -- emulation of the LEMO input
     ext_sync_i         : in std_logic;      -- sync pulse
     
     -- Power supplies for the ADC
     adc_1v8_shdn_n_o   : out std_logic;
     adc_m5v_shdn_n_o   : out std_logic;
     adc_5v_en_n_o      : out std_logic;
     
     -- FielDrive
     fd_rxcdn_i         : in    std_logic;
     fd_rxd_i           : in    std_logic;
     fd_txer_i          : in    std_logic;
     fd_wdgn_i          : in    std_logic;
     fd_rstn_o          : out   std_logic;
     fd_txck_o          : out   std_logic;
     fd_txd_o           : out   std_logic;
     fd_txena_o         : out   std_logic;

    -- WISHBONE classic bus interface
     wb_adr_i           : in    std_logic_vector(g_span-1 downto 0);
     wb_dat_i           : in    std_logic_vector(g_width-1 downto 0);
     wb_stb_i           : in    std_logic;
     wb_sel_i           : in    std_logic_vector(3 downto 0);
     wb_we_i            : in    std_logic;
     wb_cyc_i           : in    std_logic;
     wb_stall_o         : out   std_logic;
     wb_ack_o           : out   std_logic;
     wb_dat_o           : out   std_logic_vector(g_width-1 downto 0);

     -- Aux
     aux_o              : out std_logic_vector(7 downto 0));   -- group of 8 aux signals to pass to the higher levels
  end fmc_masterFIP_core;


--=================================================================================================
--                                    architecture declaration
--=================================================================================================
architecture rtl of fmc_masterFIP_core is

  -- resets
  constant c_RST_UNLOCK_VALUE : std_logic_vector(15 downto 0)    := x"cafe";
  signal rst_n, core_rst, core_rst_n, rx_rst, rx_rst_p           : std_logic;
  signal core_rstn_tmp, fd_rstn_tmp, rstn_lock_wr, fd_host_rstn  : std_logic;
  signal core_host_rst, fd_host_rst, core_host_rstn              : std_logic;
  signal rstn_lock                                               : std_logic_vector(15 downto 0);
  -- ext pulse
  signal ext_sync_p                                              : std_logic;
  -- counters
  signal macrocyc_lgth, turnar_lgth, silen_lgth                  : std_logic_vector(30 downto 0);
  signal macrocyc_host_load, turnar_host_load, silen_host_load   : std_logic;
  signal macrocyc_host_load_p, turnar_host_load_p                : std_logic;
  signal silen_host_load_p                                       : std_logic;
  signal macrocyc_cnt, turnar_cnt, silen_cnt                     : std_logic_vector(30 downto 0);
  signal macrocyc_cnt_zero_p                                     : std_logic; 
  -- tx
  signal tx_completed_p, tx_completed, tx_start, tx_start_p      : std_logic;
  signal tx_rst, tx_rst_p, fd_txd, fd_txck                       : std_logic;
  signal tx_frame                                                : tx_frame_t;
  signal tx_ctrl_byte                                            : std_logic_vector(7 downto 0);
  signal tx_bytes_num, tx_byte_index                             : std_logic_vector(8 downto 0); -- up to 263 bytes
  -- rx
  signal rx_fss_received_p, rx_fss_received, rx_fss_crc_fes_ok_p : std_logic;
  signal rx_frame_ok_p, rx_frame_ok, rx_frame_ok_p_d1            : std_logic;
  signal rx_crc_wrong_p, rx_crc_wrong, rx_byte_ready_p, fd_txena : std_logic;
  signal rx_frame                                                : rx_frame_t;
  signal rx_ctrl_byte, rx_byte                                   : std_logic_vector(7 downto 0);
  signal rx_byte_index, rx_bytes_num                             : std_logic_vector(8 downto 0); -- up to 263 bytes
  
  -- chipscope
   -- component chipscope_ila
     -- port (
       -- CONTROL : inout std_logic_vector(35 downto 0);
       -- CLK     : in    std_logic;
       -- TRIG0   : in    std_logic_vector(31 downto 0);
       -- TRIG1   : in    std_logic_vector(31 downto 0);
       -- TRIG2   : in    std_logic_vector(31 downto 0);
       -- TRIG3   : in    std_logic_vector(31 downto 0));
   -- end component;
   -- component chipscope_icon
     -- port (CONTROL0 : inout std_logic_vector(35 downto 0));
   -- end component;
   -- signal CONTROL : std_logic_vector(35 downto 0);
   -- signal CLK     : std_logic;
   -- signal TRIG0   : std_logic_vector(31 downto 0);
   -- signal TRIG1   : std_logic_vector(31 downto 0);
   -- signal TRIG2   : std_logic_vector(31 downto 0);
   -- signal TRIG3   : std_logic_vector(31 downto 0);

  
--=================================================================================================
--                                       architecture begin
--=================================================================================================
begin

---------------------------------------------------------------------------------------------------
--                                           REGISTERS                                           --
---------------------------------------------------------------------------------------------------
  cmp_masterfip_csr: fmc_masterfip_csr
  port map(
    -- wishbone
    rst_n_i                           => rst_n,
    clk_sys_i                         => clk_i,
    wb_adr_i                          => wb_adr_i(9 downto 2),
    wb_dat_i                          => wb_dat_i,
    wb_dat_o                          => wb_dat_o,
    wb_cyc_i                          => wb_cyc_i,
    wb_sel_i                          => wb_sel_i,
    wb_stb_i                          => wb_stb_i,
    wb_we_i                           => wb_we_i,
    wb_ack_o                          => wb_ack_o,
    wb_stall_o                        => wb_stall_o,
    -- resets
    mf_rstn_core_o                    => core_rstn_tmp, -- reset has to be confirmed with lock word mf_rstn_lock_o 
    mf_rstn_fd_o                      => fd_rstn_tmp,   -- reset has to be confirmed with lock word mf_rstn_lock_o
    mf_rstn_lock_o                    => rstn_lock,
    mf_rstn_lock_wr_o                 => rstn_lock_wr,
    -- adc supplies
    mf_adc_1v8_shdn_n_o               => adc_1v8_shdn_n_o,
    mf_adc_m5v_shdn_n_o               => adc_m5v_shdn_n_o,
    mf_adc_5v_en_n_o                  => adc_5v_en_n_o,
    -- external sync pulse
    mf_ext_sync_term_en_o             => ext_sync_term_en_o,
    mf_ext_sync_dir_o                 => ext_sync_dir_o,
    mf_ext_sync_oe_o                  => ext_sync_oe_o,
    mf_ext_sync_tst_n_o               => ext_sync_tst_n_o,
    mf_ext_sync_tstamp_i              => "00000000000000000000000000000000", -- external sync tstamp
    -- bus termination
    mf_bus_term_en_n_o                => bus_term_en_n_o,
    -- bus speed
    mf_speed_i(1)                     => speed_b1_i,
    mf_speed_i(0)                     => speed_b0_i,    
    -- counters outputs
    mf_macrocyc_lgth_o                => macrocyc_lgth,
    mf_turnar_lgth_o                  => turnar_lgth,
    mf_silen_lgth_o                   => silen_lgth,
    mf_macrocyc_start_o               => macrocyc_host_load, -- optional counters loading through the host
    mf_turnar_start_o                 => turnar_host_load,
    mf_silen_start_o                  => silen_host_load,
    -- counters inputs   
    mf_macrocyc_time_cnt_i            => (macrocyc_cnt_zero_p or macrocyc_host_load_p) & macrocyc_cnt,
    mf_turnar_time_cnt_i              => (tx_completed_p or turnar_host_load_p) & turnar_cnt,
    mf_silen_time_cnt_i               => (tx_completed_p or silen_host_load_p) & silen_cnt,
    -- tx control
    mf_tx_ctrl_rst_o                  => tx_rst,         -- tx rst; then converted to 1-tick-long pulse
    mf_tx_ctrl_start_o                => tx_start,       -- tx start serializing a frame; then converted to 1-tick-long pulse
    mf_tx_ctrl_bytes_num_o            => tx_bytes_num,   -- number of bytes to serialize; sampled upon tx_start
    -- tx status
    mf_tx_stat_stop_i                 => tx_completed,   -- 16x25ns long pulse upon the end of delivery of a frame
    mf_tx_stat_byte_index_i           => tx_byte_index,  -- index of current byte being serialized
    mf_tx_stat_ena_i                  => fd_txena,       -- FIELDRIVE txena,    signal generated by the tx_serializer
    mf_fd_wdgn_i                      => fd_wdgn_i,      -- FIELDRIVE watchdog, signal comes from the FIELDRIVE
    mf_fd_txer_i                      => fd_txer_i,      -- FIELDRIVE txerr,    signal comes from the FIELDRIVE
    -- rx control/status
    mf_rx_ctrl_rst_o                  => rx_rst,         -- note: this is not a pulse
    -- rx status
    mf_rx_stat_pream_ok_i             => rx_fss_received,
    mf_rx_stat_frame_ok_i             => rx_frame_ok,
    mf_rx_stat_frame_err_i            => rx_crc_wrong,
    mf_rx_stat_frame_err_code_i       => "000",
    mf_rx_stat_bytes_num_i            => rx_bytes_num,
    -- rx data
    mf_rx_data_ctrl_i                 => rx_ctrl_byte,
    mf_rx_data_reg1_i                 => rx_frame(0),
    mf_rx_data_reg2_i                 => rx_frame(1),
    mf_rx_data_reg3_i                 => rx_frame(2),
    mf_rx_data_reg4_i                 => rx_frame(3),
    mf_rx_data_reg5_i                 => rx_frame(4),
    mf_rx_data_reg6_i                 => rx_frame(5),
    mf_rx_data_reg7_i                 => rx_frame(6),
    mf_rx_data_reg8_i                 => rx_frame(7),
    mf_rx_data_reg9_i                 => rx_frame(8),
    mf_rx_data_reg10_i                => rx_frame(9),
    mf_rx_data_reg11_i                => rx_frame(10),
    mf_rx_data_reg12_i                => rx_frame(11),
    mf_rx_data_reg13_i                => rx_frame(12),
    mf_rx_data_reg14_i                => rx_frame(13),
    mf_rx_data_reg15_i                => rx_frame(14),
    mf_rx_data_reg16_i                => rx_frame(15),
    mf_rx_data_reg17_i                => rx_frame(16),
    mf_rx_data_reg18_i                => rx_frame(17),
    mf_rx_data_reg19_i                => rx_frame(18),
    mf_rx_data_reg20_i                => rx_frame(19),
    mf_rx_data_reg21_i                => rx_frame(20),
    mf_rx_data_reg22_i                => rx_frame(21),
    mf_rx_data_reg23_i                => rx_frame(22),
    mf_rx_data_reg24_i                => rx_frame(23),
    mf_rx_data_reg25_i                => rx_frame(24),
    mf_rx_data_reg26_i                => rx_frame(25),
    mf_rx_data_reg27_i                => rx_frame(26),
    mf_rx_data_reg28_i                => rx_frame(27),
    mf_rx_data_reg29_i                => rx_frame(28),
    mf_rx_data_reg30_i                => rx_frame(29),
    mf_rx_data_reg31_i                => rx_frame(30),
    mf_rx_data_reg32_i                => rx_frame(31),
    mf_rx_data_reg33_i                => rx_frame(32),
    mf_rx_data_reg34_i                => rx_frame(33),
    mf_rx_data_reg35_i                => rx_frame(34),
    mf_rx_data_reg36_i                => rx_frame(35),
    mf_rx_data_reg37_i                => rx_frame(36),
    mf_rx_data_reg38_i                => rx_frame(37),
    mf_rx_data_reg39_i                => rx_frame(38),
    mf_rx_data_reg40_i                => rx_frame(39),
    mf_rx_data_reg41_i                => rx_frame(40),
    mf_rx_data_reg42_i                => rx_frame(41),
    mf_rx_data_reg43_i                => rx_frame(42),
    mf_rx_data_reg44_i                => rx_frame(43),
    mf_rx_data_reg45_i                => rx_frame(44),
    mf_rx_data_reg46_i                => rx_frame(45),
    mf_rx_data_reg47_i                => rx_frame(46),
    mf_rx_data_reg48_i                => rx_frame(47),
    mf_rx_data_reg49_i                => rx_frame(48),
    mf_rx_data_reg50_i                => rx_frame(49),
    mf_rx_data_reg51_i                => rx_frame(50),
    mf_rx_data_reg52_i                => rx_frame(51),
    mf_rx_data_reg53_i                => rx_frame(52),
    mf_rx_data_reg54_i                => rx_frame(53),
    mf_rx_data_reg55_i                => rx_frame(54),
    mf_rx_data_reg56_i                => rx_frame(55),
    mf_rx_data_reg57_i                => rx_frame(56),
    mf_rx_data_reg58_i                => rx_frame(57),
    mf_rx_data_reg59_i                => rx_frame(58),
    mf_rx_data_reg60_i                => rx_frame(59),
    mf_rx_data_reg61_i                => rx_frame(60),
    mf_rx_data_reg62_i                => rx_frame(61),
    mf_rx_data_reg63_i                => rx_frame(62),
    mf_rx_data_reg64_i                => rx_frame(63),
    mf_rx_data_reg65_i                => rx_frame(64),
    mf_rx_data_reg66_i                => rx_frame(65),    
    -- tx data
    mf_tx_data_ctrl_o                 => tx_ctrl_byte,
    mf_tx_data_reg1_o                 => tx_frame(0),
    mf_tx_data_reg2_o                 => tx_frame(1),
    mf_tx_data_reg3_o                 => tx_frame(2),
    mf_tx_data_reg4_o                 => tx_frame(3),
    mf_tx_data_reg5_o                 => tx_frame(4),
    mf_tx_data_reg6_o                 => tx_frame(5),
    mf_tx_data_reg7_o                 => tx_frame(6),
    mf_tx_data_reg8_o                 => tx_frame(7),
    mf_tx_data_reg9_o                 => tx_frame(8),
    mf_tx_data_reg10_o                => tx_frame(9),
    mf_tx_data_reg11_o                => tx_frame(10),
    mf_tx_data_reg12_o                => tx_frame(11),
    mf_tx_data_reg13_o                => tx_frame(12),
    mf_tx_data_reg14_o                => tx_frame(13),
    mf_tx_data_reg15_o                => tx_frame(14),
    mf_tx_data_reg16_o                => tx_frame(15),
    mf_tx_data_reg17_o                => tx_frame(16),
    mf_tx_data_reg18_o                => tx_frame(17),
    mf_tx_data_reg19_o                => tx_frame(18),
    mf_tx_data_reg20_o                => tx_frame(19),
    mf_tx_data_reg21_o                => tx_frame(20),
    mf_tx_data_reg22_o                => tx_frame(21),
    mf_tx_data_reg23_o                => tx_frame(22),
    mf_tx_data_reg24_o                => tx_frame(23),
    mf_tx_data_reg25_o                => tx_frame(24),
    mf_tx_data_reg26_o                => tx_frame(25),
    mf_tx_data_reg27_o                => tx_frame(26),
    mf_tx_data_reg28_o                => tx_frame(27),
    mf_tx_data_reg29_o                => tx_frame(28),
    mf_tx_data_reg30_o                => tx_frame(29),
    mf_tx_data_reg31_o                => tx_frame(30),
    mf_tx_data_reg32_o                => tx_frame(31),
    mf_tx_data_reg33_o                => tx_frame(32),
    mf_tx_data_reg34_o                => tx_frame(33),
    mf_tx_data_reg35_o                => tx_frame(34),
    mf_tx_data_reg36_o                => tx_frame(35),
    mf_tx_data_reg37_o                => tx_frame(36),
    mf_tx_data_reg38_o                => tx_frame(37),
    mf_tx_data_reg39_o                => tx_frame(38),
    mf_tx_data_reg40_o                => tx_frame(39),
    mf_tx_data_reg41_o                => tx_frame(40),
    mf_tx_data_reg42_o                => tx_frame(41),
    mf_tx_data_reg43_o                => tx_frame(42),
    mf_tx_data_reg44_o                => tx_frame(43),
    mf_tx_data_reg45_o                => tx_frame(44),
    mf_tx_data_reg46_o                => tx_frame(45),
    mf_tx_data_reg47_o                => tx_frame(46),
    mf_tx_data_reg48_o                => tx_frame(47),
    mf_tx_data_reg49_o                => tx_frame(48),
    mf_tx_data_reg50_o                => tx_frame(49),
    mf_tx_data_reg51_o                => tx_frame(50),
    mf_tx_data_reg52_o                => tx_frame(51),
    mf_tx_data_reg53_o                => tx_frame(52),
    mf_tx_data_reg54_o                => tx_frame(53),
    mf_tx_data_reg55_o                => tx_frame(54),
    mf_tx_data_reg56_o                => tx_frame(55),
    mf_tx_data_reg57_o                => tx_frame(56),
    mf_tx_data_reg58_o                => tx_frame(57),
    mf_tx_data_reg59_o                => tx_frame(58),
    mf_tx_data_reg60_o                => tx_frame(59),
    mf_tx_data_reg61_o                => tx_frame(60),
    mf_tx_data_reg62_o                => tx_frame(61),
    mf_tx_data_reg63_o                => tx_frame(62),
    mf_tx_data_reg64_o                => tx_frame(63),
    mf_tx_data_reg65_o                => tx_frame(64),
    mf_tx_data_reg66_o                => tx_frame(65));


    
---------------------------------------------------------------------------------------------------
--                                             resets                                            --
---------------------------------------------------------------------------------------------------
  rst_n <= not rst_i;
--  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --
  -- reset generation
  p_host_reset : process(clk_i)
  begin
    if rising_edge(clk_i) then
      if(rst_n = '0') then
        core_host_rstn   <= '0';
        fd_host_rstn     <= '0';
      else
        -- protection against accidental write
        if(rstn_lock_wr = '1' and rstn_lock = c_RST_UNLOCK_VALUE) then
          core_host_rstn <= core_rstn_tmp;
          fd_host_rstn   <= fd_rstn_tmp;
        end if;
      end if;
    end if;
  end process;
  --  --  --  --  --  --  --  --  --  --  --
  core_host_rst <= not core_host_rstn;
  fd_rstn_o   <= fd_host_rstn;
  core_rst    <= core_host_rst or rst_i; -- combination of PoR, PCIe reset, reset from WRNC
  core_rst_n  <= not core_rst;



---------------------------------------------------------------------------------------------------
--                                         external sync                                         --
---------------------------------------------------------------------------------------------------
  cmp_ext_sync_p_detect: gc_sync_ffs
  generic map(g_sync_edge => "positive")
  port map(
    clk_i    => clk_i,
    rst_n_i  => core_rst_n,
    data_i   => ext_sync_i,
    ppulse_o => ext_sync_p);

    
---------------------------------------------------------------------------------------------------
--                                       macrocycle counter                                      --
---------------------------------------------------------------------------------------------------
  cmp_macrocycle_cnt: decr_counter
  generic map(width   => 31)
  port map(
    clk_i             => clk_i,
    rst_i             => core_rst,
    counter_load_i    => macrocyc_cnt_zero_p or macrocyc_host_load_p or ext_sync_p,
    counter_top_i     => macrocyc_lgth,
    counter_o         => macrocyc_cnt,
    counter_is_zero_o => macrocyc_cnt_zero_p);

  cmp_macrocyc_host_load_p_detect: gc_sync_ffs
  generic map(g_sync_edge => "positive")
  port map(
    clk_i    => clk_i,
    rst_n_i  => core_rst_n,
    data_i   => macrocyc_host_load,
    ppulse_o => macrocyc_host_load_p);

    
---------------------------------------------------------------------------------------------------
--                                       tournaround counter                                     --
---------------------------------------------------------------------------------------------------
  cmp_turnaround_cnt: decr_counter
  generic map(width   => 31)
  port map(
    clk_i             => clk_i,
    rst_i             => core_rst,
    counter_load_i    => tx_completed_p or rx_frame_ok_p or rx_crc_wrong_p or turnar_host_load_p,
    counter_top_i     => turnar_lgth,
    counter_o         => turnar_cnt,
    counter_is_zero_o => open);

  turnar_host_load_p_detect: gc_sync_ffs
  generic map(g_sync_edge => "positive")
  port map(
    clk_i    => clk_i,
    rst_n_i  => core_rst_n,
    data_i   => turnar_host_load,
    ppulse_o => turnar_host_load_p);
    
--------------------------------------------------------------------------------------------------
--                                        silence counter                                       --
--------------------------------------------------------------------------------------------------
-- silence time: maximum amount of time that the masterFIP waits for a response frame.
-- the counting starts after an ID_DAT sent by the master (therefore loaded upon tx_completed_p)
-- or by the FES of an RP_DAT_MSG as a RP_FIN is expected afterwards
  cmp_silence_cnt: decr_counter
  generic map(width   => 31)
  port map(
    clk_i             => clk_i,
    rst_i             => core_rst,
    counter_load_i    => tx_completed_p or rx_frame_ok_p or rx_crc_wrong_p or silen_host_load_p,
    counter_top_i     => silen_lgth,
    counter_o         => silen_cnt,
    counter_is_zero_o => open);

  cmp_silen_host_load_p_detect: gc_sync_ffs
  generic map(g_sync_edge => "positive")
  port map(
    clk_i    => clk_i,
    rst_n_i  => core_rst_n,
    data_i   => silen_host_load,
    ppulse_o => silen_host_load_p);

    
--------------------------------------------------------------------------------------------------
--                                        signals for tx                                        --
--------------------------------------------------------------------------------------------------
-- edge detection on the tx_start signal
  cmp_tx_start_p_detect: gc_sync_ffs
  generic map(g_sync_edge => "positive")
  port map(
    clk_i    => clk_i,
    rst_n_i  => core_rst_n,
    data_i   => tx_start,
    ppulse_o => tx_start_p);


--------------------------------------------------------------------------------------------------
--                                        signals for rx                                        --
--------------------------------------------------------------------------------------------------
-- extension of the rx_fss_received_p so as to ensure capturing by the wrnc
  cmp_rx_extend_p : gc_extend_pulse
    generic map (
      g_width => 32)
    port map (
      clk_i      => clk_i,
      rst_n_i    => core_rst_n,
      pulse_i    => rx_fss_received_p,
      extended_o => rx_fss_received);

-- extension of the rx_frame_ok_p so as to ensure capturing by the wrnc
  cmp_rx_frame_ok_extend_p : gc_extend_pulse
    generic map (
      g_width => 32)
    port map (
      clk_i      => clk_i,
      rst_n_i    => core_rst_n,
      pulse_i    => rx_frame_ok_p_d1,
      extended_o => rx_frame_ok);

-- extension of the rx_crc_wrong_p so as to ensure capturing by the wrnc
  cmp_rx_crc_wrong_extend_p : gc_extend_pulse
    generic map (
      g_width => 32)
    port map (
      clk_i      => clk_i,
      rst_n_i    => core_rst_n,
      pulse_i    => rx_crc_wrong_p,
      extended_o => rx_crc_wrong);       

  cmp_rx_rst_pulse_detect: gc_sync_ffs
  generic map(g_sync_edge => "positive")
  port map(
    clk_i    => clk_i,
    rst_n_i  => rst_n,
    data_i   => rx_rst,
    ppulse_o => rx_rst_p);

  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --
  -- registering the number of received bytes
    p_rx_bytes_num : process(clk_i)
  begin
    if rising_edge(clk_i) then
      if(rst_n = '0') then
        rx_bytes_num       <= (others => '0');
        rx_frame_ok_p_d1   <= '0';
      else
        rx_frame_ok_p_d1   <= rx_frame_ok_p;
        if rx_frame_ok_p = '1' then
          rx_bytes_num     <= rx_byte_index;
        end if;
      end if;
    end if;
  end process;
      
---------------------------------------------------------------------------------------------------
--                                               rx                                              --
---------------------------------------------------------------------------------------------------
  cmp_masterfip_rx: masterfip_rx
  port map(
    clk_40m_i             => clk_i,
    rst_i                 => core_rst,
    speed_i               => speed_b1_i & speed_b0_i,
    fd_rxd_a_i            => fd_rxd_i,
    rx_rst_i              => rx_rst_p or fd_txena, -- reset from the WRNC or reset while transmitting
  -------------------------------------------------------------
    rx_byte_index_o       => rx_byte_index, -- current amount of bytes (counting starts after the fss;
                                            -- includes ctrl, data, crc, fes fields)
    rx_ctrl_byte_o        => rx_ctrl_byte,
    rx_frame_o            => rx_frame,
    rx_byte_o             => rx_byte,
    rx_byte_ready_p_o     => rx_byte_ready_p,
    rx_fss_crc_fes_ok_p_o => rx_frame_ok_p,
    rx_fss_received_p_o   => rx_fss_received_p,
    rx_crc_wrong_p_o      => rx_crc_wrong_p);
  -------------------------------------------------------------

  
---------------------------------------------------------------------------------------------------
--                                               tx                                              --
---------------------------------------------------------------------------------------------------

  cmp_masterfip_tx: masterfip_tx
  port map(
    clk_40m_i       => clk_i,
    rst_i           => (core_rst or tx_rst_p),
    speed_i         => speed_b1_i & speed_b0_i,
    tx_bytes_num_i  => tx_bytes_num,
    tx_start_p_i    => tx_start_p,
    tx_frame_i      => tx_frame,
    tx_ctrl_byte_i  => tx_ctrl_byte,
  -------------------------------------------------------------
    tx_byte_index_o => tx_byte_index,
    tx_end_p_o      => tx_completed_p,
    tx_data_o       => fd_txd,
    tx_enable_o     => fd_txena,
    tx_clk_o        => fd_txck);
  -------------------------------------------------------------
  
  fd_txena_o <= fd_txena;
  fd_txd_o   <= fd_txd;
  fd_txck_o  <= fd_txck;

  
--------------------------------------------------------------------------------------------------
--                                        signals for tx                                        --
-------------------------------------------------------------------------------------------------- 

-- create 1-clk-tick long pulse upon tx_rst
  cmp_tx_rst_pulse_detect: gc_sync_ffs
  generic map(g_sync_edge => "positive")
  port map(
    clk_i    => clk_i,
    rst_n_i  => rst_n,
    data_i   => tx_rst,
    ppulse_o => tx_rst_p);
    
-- extension of the tx_completed_p so as to ensure capturing by the wrnc
  cmp_tx_extend_p : gc_extend_pulse
    generic map (
      g_width => 32)
    port map (
      clk_i      => clk_i,
      rst_n_i    => core_rst_n,
      pulse_i    => tx_completed_p,
      extended_o => tx_completed);   
      
      
---------------------------------------------------------------------------------------------------
--                                              aux                                             --
---------------------------------------------------------------------------------------------------  
  aux_o(7) <= rx_fss_received_p;
  aux_o(6) <= rx_fss_received;
  aux_o(5) <= rx_frame_ok;
  aux_o(4) <= rx_byte_ready_p;
  aux_o(3) <= tx_rst;
  aux_o(2) <= tx_completed_p;
  aux_o(1) <= tx_start;
  aux_o(0) <= rx_frame_ok_p;

  
  
---------------------------------------------------------------------------------------------------
--                                           CHIPSCOPE                                           --
---------------------------------------------------------------------------------------------------   
  
   -- chipscope_ila_1 : chipscope_ila
   -- port map (
     -- CONTROL => CONTROL,
     -- CLK     => clk_i,
     -- TRIG0   => TRIG0,
     -- TRIG1   => TRIG1,
     -- TRIG2   => TRIG2,
     -- TRIG3   => TRIG3);

  -- chipscope_icon_1 : chipscope_icon
   -- port map ( CONTROL0 => CONTROL);
 
  -- TRIG0(8 downto 0)   <= tx_bytes_num;
  -- TRIG0(9)            <= tx_start;
  -- TRIG0(10)           <= tx_start_p;
  -- TRIG0(18 downto 11) <= tx_ctrl_byte;
  -- TRIG0(27 downto 19) <= tx_byte_index;
  -- TRIG0(28)           <= tx_completed_p;
  -- TRIG0(29)           <= fd_txena;
  -- TRIG0(30)           <= fd_txd;
  -- TRIG0(31)           <= fd_txck;

  -- TRIG1               <= tx_frame(0);

  -- TRIG2(8 downto 0)   <= rx_byte_index;
  -- TRIG2(9)            <= rx_fss_received_p;
  -- TRIG2(10)           <= rx_byte_ready_p;
  -- TRIG2(18 downto 11) <= rx_byte;
  -- TRIG2(26 downto 19) <= rx_ctrl_byte;
  -- TRIG2(27)           <= rx_frame_ok_p;
  -- TRIG2(28)           <= rx_crc_wrong_p;
  
  -- TRIG2(29)           <= core_rst;
  -- TRIG2(31 downto 30) <= speed_b1_i & speed_b0_i;
  
  -- TRIG3               <= rx_frame(0);
  
  --TRIG0(17)           <= tx_start_p;
  --TRIG0(18)           <= rx_fss_crc_fes_ok_p;
  --TRIG0(19)           <= start_tx_p;

  --TRIG0(21)           <= rx_fss_crc_fes_ok_p;
  --TRIG0(22)           <= rx_fss_received_p;
  --TRIG0(23)           <= rx_crc_wrong_p;


  --TRIG1(15 downto 8)  <= rx_byte_index;
  --TRIG0(16)           <= start_from_fpga;  
  --TRIG0(24 downto 17) <= retrig_nb_offset(7 downto 0);
  --TRIG0(31 downto 25) <= clk_i_cycles_offset(6 downto 0);  

  
end rtl;
----------------------------------------------------------------------------------------------------
--  architecture ends
----------------------------------------------------------------------------------------------------
